@@ -27,12 +27,12 @@
    )* }}} *)
 
 type t =
-  | Constructor of Types.constructor_description * Location.t
-    (* We attach the constructor description here so in the case of
-       disambiguated constructors we actually directly look for the type
-       path (cf. #486, #794). *)
+  | Constructor of Data_types.constructor_description * Location.t
+  (* We attach the constructor description here so in the case of
+     disambiguated constructors we actually directly look for the type
+     path (cf. #486, #794). *)
   | Expr
-  | Label of Types.label_description (* Similar to constructors. *)
+  | Label of Data_types.label_description (* Similar to constructors. *)
   | Module_path
   | Module_type
   | Patt
@@ -43,16 +43,16 @@ type t =
 val to_string : t -> string
 
 (**
-  [inspect_browse_tree lid ~cursor mbrowse] tries to provide contextual
-  information given the selected identifier, the position of the cursor and the
-  typed tree. It is used by Locate and Type_enclosing.
+   [inspect_browse_tree lid ~cursor mbrowse] tries to provide contextual
+   information given the selected identifier, the position of the cursor and the
+   typed tree. It is used by Locate and Type_enclosing.
 
-  The cursor position is used to distinguished whether a module path or an actual
-  constructor name is pointed at when the cursor is in the middle of a
-  longident, e.g. [Foo.B|ar.Constructor] (with | being the cursor).
+   The cursor position is used to distinguished whether a module path or an actual
+   constructor name is pointed at when the cursor is in the middle of a
+   longident, e.g. [Foo.B|ar.Constructor] (with | being the cursor).
 
-  FIXME: when cursor at (M.|A 3), the enclosing node returned is const 3, thus
-  breaking the context inference.
+   FIXME: when cursor at (M.|A 3), the enclosing node returned is const 3, thus
+   breaking the context inference.
 *)
 val inspect_browse_tree :
   cursor:Std.Lexing.position -> Longident.t -> Mbrowse.t list -> t option
