@@ -1,12 +1,12 @@
 (** Provides tools to lookup items in the typing environment.
 
-  Establishing the namespace of an item before looking it up in the environement
-  is necessary to prevent mixing items which have the same name but are not of
-  the same namespace. (For example the environment can contain both type named
-  `t` and a value named `t`.) *)
+    Establishing the namespace of an item before looking it up in the environement
+    is necessary to prevent mixing items which have the same name but are not of
+    the same namespace. (For example the environment can contain both type named
+    `t` and a value named `t`.) *)
 
 (** Namespaces describe in which section of the environment an item should be
-  looked for. *)
+    looked for. *)
 module Namespace : sig
   type t = Shape.Sig_component_kind.t
 
@@ -19,8 +19,8 @@ module Namespace : sig
     | `Labels
     | `Mod
     | `Modtype
-    | `This_cstr of Types.constructor_description
-    | `This_label of Types.label_description
+    | `This_cstr of Data_types.constructor_description
+    | `This_label of Data_types.label_description
     | `Type
     | `Vals ]
 
