@@ -294,7 +294,23 @@ module Utils = struct
     | CMT _ | CMTI _ -> Mconfig.cmt_path config
 end
 
-let move_to filename cmt_infos =
+let reroot_build_dir ~root path =
+  let sep =
+    try String.get Filename.dir_sep 0 with Invalid_argument _ -> '/'
+  in
+  let segments = path |> String.split_on_char ~sep in
+  let rec strip_prefix = function
+    | [] -> []
+    | "_build" :: _ as l -> l
+    | _ :: tl -> strip_prefix tl
+  in
+  match strip_prefix segments with
+  | [] -> path
+  | l ->
+    let sep = Printf.sprintf "%c" sep in
+    Filename.concat root (String.concat ~sep l)
+
+let move_to (config : Mconfig.t) filename cmt_infos =
   let digest =
     (* [None] only for packs, and we wouldn't have a trie if the cmt was for a
        pack. *)
@@ -302,6 +318,14 @@ let move_to filename cmt_infos =
       Filename.concat cmt_infos.Cmt_format.cmt_builddir
         (Option.get cmt_infos.cmt_sourcefile)
     in
+    let sourcefile_in_builddir =
+      (* This workaround is meant to fix issues with Dune's BUILD_PREFIX_MAP It
+         will not work when the [_build] folder is not located at the source
+         root. See [#1934](https://github.com/ocaml/merlin/issues/1934). *)
+      match config.merlin.source_root with
+      | None -> sourcefile_in_builddir
+      | Some root -> reroot_build_dir ~root sourcefile_in_builddir
+    in
     match
       sourcefile_in_builddir |> String.split_on_char ~sep:'.' |> List.rev
     with
@@ -332,7 +356,7 @@ let load_cmt ~config ?(with_fallback = true) comp_unit =
     let cmt_infos = (Cmt_cache.read path).cmt_infos in
     let source_file = cmt_infos.cmt_sourcefile in
     let source_file = Option.value ~default:"*pack*" source_file in
-    move_to path cmt_infos;
+    move_to config.mconfig path cmt_infos;
     Ok (source_file, cmt_infos)
   | None -> Error ()
 
@@ -386,12 +410,13 @@ let find_source ~config loc =
     | Some s -> s
   in
   log ~title:"find_source" "initial path: %S" initial_path;
-  let dir = Filename.dirname initial_path in
-  let dir =
+  let canonical_dir_for_file file =
+    let raw_dir = Filename.dirname file in
     match config.Mconfig.query.directory with
-    | "" -> dir
-    | cwd -> Misc.canonicalize_filename ~cwd dir
+    | "" -> raw_dir
+    | cwd -> Misc.canonicalize_filename ~cwd raw_dir
   in
+  let dir = canonical_dir_for_file initial_path in
   match Utils.find_all_matches ~config ~with_fallback file with
   | [] ->
     log ~title:"find_source" "failed to find %S in source path (fallback = %b)"
@@ -410,22 +435,28 @@ let find_source ~config loc =
       ~title:(sprintf "find_source(%s)" filename)
       "multiple matches in the source path : %s"
       (String.concat ~sep:" , " files);
-    try
+    let files_matching_digest =
       match File_switching.source_digest () with
       | None ->
         log ~title:"find_source"
           "... no source digest available to select the right one";
-        raise Not_found
+        []
       | Some digest ->
         log ~title:"find_source"
           "... trying to use source digest to find the right one";
         log ~title:"find_source" "Source digest: %s" (Digest.to_hex digest);
-        Found
-          (List.find files ~f:(fun f ->
-               let fdigest = Digest.file f in
-               log ~title:"find_source" "  %s (%s)" f (Digest.to_hex fdigest);
-               fdigest = digest))
-    with Not_found -> (
+
+        List.filter files ~f:(fun f ->
+            let fdigest = Digest.file f in
+            log ~title:"find_source" "  %s (%s)" f (Digest.to_hex fdigest);
+            fdigest = digest)
+    in
+    match files_matching_digest with
+    | [ file ] ->
+      log ~title:"find_source" "... found exactly one file with matching digest";
+      Found file
+    | [] -> (
+      log ~title:"find_source" "... found no files with matching digest";
       log ~title:"find_source" "... using heuristic to select the right one";
       log ~title:"find_source" "we are looking for a file named %s in %s" fname
         dir;
@@ -462,7 +493,66 @@ let find_source ~config loc =
       match lst with
       | (i1, _) :: (i2, _) :: _ when i1 = i2 -> Multiple_matches files
       | (_, s) :: _ -> Found s
-      | _ -> assert false))
+      | _ -> assert false)
+    | files_matching_digest ->
+      log ~title:"find_source" "... found multiple files with matching digest";
+      log ~title:"find_source"
+        "... using directory heuristic to choose the best one";
+      (* Give each source file a score that represents how close its path is to the
+           target path (the path of the build artifact) and then choose the source file
+           with the highest score.
+
+           The score of a source file is the longest tail of the path of the its
+           directory that is a subpath of the target path. This is premised on build
+           systems liking to put artifacts in paths that are similar to the source path.
+           i.e., dune may put the cmt for foo/bar/baz.ml in
+           _build/default/foo/bar/.bar.objs/byte/bar__Baz.cmt, so we want to use that
+           shared foo/bar in the path to disambiguate.
+
+           ex:
+             source file: /a/b/c/d/e/f.ml
+             target path: /a/b/c/_build/default/d/e/artifacts/f.cmi
+             score: 2, because /a/b/c/d/e is the source file's directory, and d/e is
+                    the longest tail of it that is a subpath of the target path. *)
+      let score_file source_file =
+        (* This is technically quadratic, but
+             a) most file paths are short
+             b) in the common case, this is linear because common_prefix_len
+                will usually fail on the first loop
+             c) this isn't a hot path - this is only for the uncommon case where there are
+                two identical files
+             So the stars would need to align for this to cause performance problems *)
+        let target_dir = dir in
+        let source_dir = canonical_dir_for_file source_file in
+        let target_dir_rev = target_dir |> Misc.split_path |> List.rev in
+        let source_dir_rev = source_dir |> Misc.split_path |> List.rev in
+        let rec common_prefix_len a b =
+          match (a, b) with
+          | [], _ | _, [] -> 0
+          | a_hd :: a_tl, b_hd :: b_tl ->
+            if String.equal a_hd b_hd then 1 + common_prefix_len a_tl b_tl
+            else 0
+        in
+        let rec candidates = function
+          | [] -> []
+          | _ :: tl as curr -> curr :: candidates tl
+        in
+        candidates target_dir_rev
+        |> List.map ~f:(common_prefix_len source_dir_rev)
+        |> List.max_elt ~cmp:Int.compare
+        |> Option.value ~default:0
+      in
+      let files_matching_digest_with_scores =
+        List.map files_matching_digest ~f:(fun file -> (file, score_file file))
+      in
+      (* get the max *)
+      let best_file, _best_score =
+        List.max_elt files_matching_digest_with_scores
+          ~cmp:(fun (_, a) (_, b) -> Int.compare a b)
+        |> Option.get
+        (* theres at least one element, so this is never None *)
+      in
+      Found best_file)
 
 (* Well, that's just another hack.
    [find_source] doesn't like the "-o" option of the compiler. This hack handles
@@ -554,14 +644,17 @@ let find_loc_of_comp_unit ~config uid comp_unit =
     log ~title "Failed to load the CU's cmt";
     `None
 
-let find_loc_of_uid ~config ~local_defs ~ident ?fallback (uid : Shape.Uid.t) =
+let find_loc_of_uid ~config ~local_defs ?ident ?fallback (uid : Shape.Uid.t) =
   let find_loc_of_item ~comp_unit =
-    match (find_loc_of_item ~config ~local_defs uid comp_unit, fallback) with
-    | Some { loc; txt }, _ when String.equal txt ident ->
+    match
+      (find_loc_of_item ~config ~local_defs uid comp_unit, fallback, ident)
+    with
+    | Some { loc; txt }, _, Some ident when String.equal txt ident ->
       (* Checking the ident prevent returning nonsensical results when some uid
          were swaped but the cmt files were not rebuilt. *)
       Some (uid, loc)
-    | (Some _ | None), Some fallback ->
+    | Some { loc; _ }, _, None -> Some (uid, loc)
+    | (Some _ | None), Some fallback, _ ->
       find_loc_of_item ~config ~local_defs fallback comp_unit
       |> Option.map ~f:(fun { Location.loc; _ } -> (fallback, loc))
     | _ -> None
@@ -604,7 +697,7 @@ let find_definition_uid ~config ~env ~(decl : Env_lookup.item) path =
           ~with_fallback:false unit_name
       with
       | Ok (filename, cmt_infos) ->
-        move_to filename cmt_infos;
+        move_to config.mconfig filename cmt_infos;
         log ~title:"read_unit_shape" "shapes loaded for %s" unit_name;
         cmt_infos.cmt_impl_shape
       | Error () ->
@@ -639,7 +732,7 @@ let rec uid_of_result ~traverse_aliases = function
   | Approximated _ | Unresolved _ | Internal_error_missing_uid -> (None, true)
 
 (** This is the main function here *)
-let from_path ~config ~env ~local_defs ~decl path =
+let from_path ~config ~env ~local_defs ~decl ?ident:_ path =
   let title = "from_path" in
   let unalias (decl : Env_lookup.item) =
     if not config.traverse_aliases then (path, decl.uid)
@@ -686,11 +779,14 @@ let from_path ~config ~env ~local_defs ~decl path =
   in
   (* Step 2:  Uid => Location *)
   let loc =
-    let ident = Path.last path in
+    let ident =
+      (* TODO it might not be useful to check the ident without impl_uid *)
+      Path.last path
+    in
     match impl_uid with
     | Some impl_uid ->
       find_loc_of_uid ~config ~local_defs ~ident ~fallback:uid impl_uid
-    | None -> find_loc_of_uid ~config ~local_defs ~ident uid
+    | None -> find_loc_of_uid ~config ~local_defs uid
   in
   let loc =
     match loc with
@@ -726,7 +822,9 @@ let from_longident ~config ~env ~local_defs nss ident =
   in
   match Env_lookup.by_longident nss ident env with
   | None -> `Not_in_env str_ident
-  | Some (path, decl) -> from_path ~config ~env ~local_defs ~decl path
+  | Some (path, decl) ->
+    let ident = Longident.last ident in
+    from_path ~config ~env ~local_defs ~decl ~ident path
 
 let from_path ~config ~env ~local_defs ~namespace path =
   File_switching.reset ();
