@@ -16,18 +16,18 @@ let rec normalize_path env path =
   match Env.find_type path env with
   | exception Not_found -> path
   | decl -> (
-    match decl.Types.type_manifest with
-    | Some body
-      when decl.Types.type_private = Asttypes.Public
-           ||
-           match decl.Types.type_kind with
-           | Types.Type_abstract _ -> false
-           | _ -> true -> begin
-      match Types.get_desc body with
-      | Types.Tconstr (path, _, _) -> normalize_path env path
-      | _ -> path
-    end
-    | _ -> path)
+      match decl.Types.type_manifest with
+      | Some body
+        when decl.Types.type_private = Asttypes.Public
+             ||
+             match decl.Types.type_kind with
+             | Types.Type_abstract _ -> false
+             | _ -> true -> begin
+          match Types.get_desc body with
+          | Types.Tconstr (path, _, _) -> normalize_path env path
+          | _ -> path
+        end
+      | _ -> path)
 
 let match_query env query t =
   let cost = ref 0 in
@@ -52,7 +52,8 @@ let match_query env query t =
       decr pos_fun;
       traverse neg neg_fun pos pos_fun t2;
       traverse pos pos_fun neg neg_fun t1
-    | Types.Ttuple ts -> List.iter ~f:(traverse neg neg_fun pos pos_fun) ts
+    | Types.Ttuple ts ->
+      List.iter ~f:(fun (_, t) -> traverse neg neg_fun pos pos_fun t) ts
     | Types.Tvar _ | Types.Tunivar _ -> decr cost (* Favor polymorphic defs *)
     | _ -> ()
   in
@@ -106,17 +107,17 @@ let directories ~global_modules env =
       match md.Types.md_type with
       | Types.Mty_alias _ -> l
       | _ ->
-        let lident = Longident.Ldot (lident, name) in
-        Trie (name, lident, lazy (explore lident env)) :: l
+        let lident = Longident.Ldot (lident, Location.mknoloc name) in
+        Trie (name, lident, lazy (explore (Location.mknoloc lident) env)) :: l
     in
-    Env.fold_modules add_module (Some lident) env []
+    Env.fold_modules add_module (Some lident.txt) env []
   in
   List.fold_left
     ~f:(fun l name ->
-      let lident = Longident.Lident name in
-      match Env.find_module_by_name lident env with
-      | exception _ -> l
-      | _ -> Trie (name, lident, lazy (explore lident env)) :: l)
+        let lident = Longident.Lident name in
+        match Env.find_module_by_name lident env with
+        | exception _ -> l
+        | _ -> Trie (name, lident, lazy (explore (Location.mknoloc lident) env)) :: l)
     ~init:[] global_modules
 (*Env.fold_modules (fun name _ _ l ->
     ignore (seen name);
@@ -128,9 +129,9 @@ let execute_query query env dirs =
   let direct dir acc =
     Env.fold_values
       (fun _ path desc acc ->
-        match match_query env query desc.Types.val_type with
-        | Some cost -> (cost, path, desc) :: acc
-        | None -> acc)
+         match match_query env query desc.Types.val_type with
+         | Some cost -> (cost, path, desc) :: acc
+         | None -> acc)
       dir env acc
   in
   let rec recurse acc (Trie (_, dir, children)) =
@@ -152,15 +153,15 @@ let execute_query query env dirs =
 let execute_query_as_type_search ?(limit = 100) ~env ~query ~modules () =
   execute_query query env modules
   |> List.map ~f:(fun (cost, path, desc) ->
-         let name =
-           Printtyp.wrap_printing_env env @@ fun () ->
-           let path = Out_type.rewrite_double_underscore_paths env path in
-           Format.asprintf "%a" Printtyp.path path
-         in
-         let doc = None in
-         let loc = desc.Types.val_loc in
-         let typ = desc.Types.val_type in
-         let constructible = Type_search.make_constructible name typ in
-         Query_protocol.{ cost; name; typ; loc; doc; constructible })
+      let name =
+        Printtyp.wrap_printing_env env @@ fun () ->
+        let path = Out_type.rewrite_double_underscore_paths env path in
+        Format.asprintf "%a" Printtyp.path path
+      in
+      let doc = None in
+      let loc = desc.Types.val_loc in
+      let typ = desc.Types.val_type in
+      let constructible = Type_search.make_constructible name typ in
+      Query_protocol.{ cost; name; typ; loc; doc; constructible })
   |> List.sort ~cmp:Type_search.compare_result
   |> List.take_n limit
