@@ -38,7 +38,7 @@ type type_info =
   | Modtype of Env.t * Types.module_type
   | Type of Env.t * Types.type_expr
   | Type_decl of Env.t * Ident.t * Types.type_declaration
-  | Type_constr of Env.t * Types.constructor_description
+  | Type_constr of Env.t * Data_types.constructor_description
   | String of string
 
 type typed_enclosings =
