@@ -27,7 +27,7 @@ let let_bound_vars bindings =
       match value_binding.Typedtree.vb_pat.pat_desc with
       | Tpat_var (id, loc, _) -> Some (id, loc)
       | Typedtree.Tpat_any
-      | Typedtree.Tpat_alias (_, _, _, _)
+      | Typedtree.Tpat_alias (_, _, _, _, _)
       | Typedtree.Tpat_constant _ | Typedtree.Tpat_tuple _
       | Typedtree.Tpat_construct (_, _, _, _)
       | Typedtree.Tpat_variant (_, _, _)
@@ -67,7 +67,7 @@ let pat_var_id_and_loc = function
   | _ -> None
 
 let pat_alias_pat_id_and_loc = function
-  | Typedtree.{ pat_desc = Tpat_alias (pat, id, loc, _); _ } ->
+  | Typedtree.{ pat_desc = Tpat_alias (pat, id, loc, _, _); _ } ->
     Some (pat, id, loc)
   | _ -> None
 
