@@ -20,12 +20,11 @@ let add_root ~root (lid : Longident.t Location.loc) =
     }
 
 let merge m m' =
-  Shape.Uid.Map.union
-    (fun _uid locs locs' -> Some (Lid_set.union locs locs'))
-    m m'
+  Uid_map.union (fun _uid locs locs' -> Some (Lid_set.union locs locs')) m m'
 
 let add_one uid lid map =
-  Shape.Uid.Map.update uid
+  let lid = Lid.of_lid lid in
+  Uid_map.update uid
     (function
       | None -> Some (Lid_set.singleton lid)
       | Some set -> Some (Lid_set.add lid set))
@@ -143,8 +142,7 @@ let index_of_cmt ~into ~root ~rewrite_root ~build_path ~do_not_use_cmt_loadpath
         let map_update uid =
           Uid_map.update uid (function
             | None -> Some union
-            | Some union' ->
-              Some (Union_find.union ~f:Uid_set.union union' union))
+            | Some union' -> Some (Union_find.union union' union))
         in
         acc |> map_update uid1 |> map_update uid2)
       into.related_uids cmt_declaration_dependencies
@@ -163,7 +161,7 @@ let merge_index ~store_shapes ~into index =
   let stats = Stats.union (fun _ f1 _f2 -> Some f1) into.stats index.stats in
   let related_uids =
     Uid_map.union
-      (fun _ a b -> Some (Union_find.union ~f:Uid_set.union a b))
+      (fun _ a b -> Some (Union_find.union a b))
       index.related_uids into.related_uids
   in
   if store_shapes then
@@ -174,12 +172,12 @@ let from_files ~store_shapes ~output_file ~root ~rewrite_root ~build_path
     ~do_not_use_cmt_loadpath files =
   Log.debug "Debug log is enabled";
   let initial_index =
-    { defs = Shape.Uid.Map.empty;
-      approximated = Shape.Uid.Map.empty;
+    { defs = Uid_map.empty ();
+      approximated = Uid_map.empty ();
       cu_shape = Hashtbl.create 64;
       stats = Stats.empty;
       root_directory = root;
-      related_uids = Uid_map.empty
+      related_uids = Uid_map.empty ()
     }
   in
   let final_index =
@@ -193,7 +191,7 @@ let from_files ~store_shapes ~output_file ~root ~rewrite_root ~build_path
             ~do_not_use_cmt_loadpath cmt_item.cmt_infos
         | exception _ -> (
           match read ~file with
-          | Index index -> merge_index ~store_shapes index ~into
+          | Index index -> merge_index ~store_shapes ~into index
           | _ ->
             Log.error "Unknown file type: %s" file;
             exit 1))
