@@ -79,7 +79,7 @@ type node =
   | Method_call of expression * meth * Location.t
   | Record_field of
       [ `Expression of expression | `Pattern of pattern ]
-      * Types.label_description
+      * Data_types.label_description
       * Longident.t Location.loc
   | Module_binding_name of module_binding
   | Module_declaration_name of module_declaration
@@ -221,8 +221,8 @@ let has_attr ~name node =
   let attrs = node_attributes node in
   List.exists
     ~f:(fun a ->
-      let str, _ = Ast_helper.Attr.as_tuple a in
-      str.Location.txt = name)
+        let str, _ = Ast_helper.Attr.as_tuple a in
+        str.Location.txt = name)
     attrs
 
 let node_merlin_loc loc0 node =
@@ -324,19 +324,21 @@ let of_pattern_desc (type k) (desc : k pattern_desc) =
   match desc with
   | Tpat_any | Tpat_var _ | Tpat_constant _ | Tpat_variant (_, None, _) ->
     id_fold
-  | Tpat_alias (p, _, _, _)
+  | Tpat_alias (p, _, _, _, _)
   | Tpat_variant (_, Some p, _)
   | Tpat_lazy p
   | Tpat_exception p -> of_pattern p
   | Tpat_value p -> of_pattern (p :> value general_pattern)
-  | Tpat_tuple ps | Tpat_construct (_, _, ps, None) | Tpat_array ps ->
+  | Tpat_tuple ps ->
+    list_fold of_pattern (List.map ~f:snd ps) (* todo handle labels *)
+  | Tpat_construct (_, _, ps, None) | Tpat_array (_, ps) ->
     list_fold of_pattern ps
   | Tpat_construct (_, _, ps, Some (_, ct)) ->
     list_fold of_pattern ps ** of_core_type ct
   | Tpat_record (ls, _) ->
     list_fold
       (fun (lid_loc, desc, p) ->
-        of_pat_record_field p lid_loc desc ** of_pattern p)
+         of_pat_record_field p lid_loc desc ** of_pattern p)
       ls
   | Tpat_or (p1, p2, _) -> of_pattern p1 ** of_pattern p2
 
@@ -356,14 +358,16 @@ let rec of_expression_desc loc = function
   | Texp_apply (e, ls) ->
     of_expression e
     ** list_fold
-         (function
-           | _, None -> id_fold
-           | _, Some e -> of_expression e)
-         ls
+      (function
+        | _, Omitted () -> id_fold
+        | _, Arg e -> of_expression e)
+      ls
   | Texp_match (e, cs, vs, _) ->
     of_expression e ** list_fold of_case cs ** list_fold of_case vs
   | Texp_try (e, cs, _) -> of_expression e ** list_fold of_case cs
-  | Texp_tuple es | Texp_construct (_, _, es) | Texp_array es ->
+  | Texp_tuple es ->
+    list_fold of_expression (List.map ~f:snd es) (* todo labels ? *)
+  | Texp_construct (_, _, es) | Texp_array (_, es) ->
     list_fold of_expression es
   | Texp_variant (_, Some e)
   | Texp_assert (e, _)
@@ -378,6 +382,7 @@ let rec of_expression_desc loc = function
         of_exp_record_field e lid_loc desc ** of_expression e
     in
     array_fold fold_field fields
+  | Texp_atomic_loc (e, _, _) -> of_expression e
   | Texp_field (e, lid_loc, lbl) ->
     of_expression e ** of_exp_record_field e lid_loc lbl
   | Texp_setfield (e1, lid_loc, lbl, e2) ->
@@ -413,7 +418,8 @@ let rec of_expression_desc loc = function
        the patterns patN when they are tuples themselves. *)
     let rec flatten_patterns ~size acc pat =
       match pat.pat_desc with
-      | Tpat_tuple [ tuple; pat ] when size > 0 ->
+      | Tpat_tuple [ (_, tuple); (_, pat) ] when size > 0 ->
+        (* todo labeled tuples ? *)
         flatten_patterns ~size:(size - 1) (pat :: acc) tuple
       | _ -> List.rev (pat :: acc)
     in
@@ -443,8 +449,8 @@ and of_class_expr_desc = function
   | Tcl_apply (ce, es) ->
     list_fold
       (function
-        | _, None -> id_fold
-        | _, Some e -> of_expression e)
+        | _, Omitted () -> id_fold
+        | _, Arg e -> of_expression e)
       es
     ** app (Class_expr ce)
   | Tcl_let (_, vbs, es, ce) ->
@@ -532,13 +538,15 @@ and of_core_type_desc = function
   | Ttyp_any | Ttyp_var _ -> id_fold
   | Ttyp_open (_, _, ct) -> of_core_type ct
   | Ttyp_arrow (_, ct1, ct2) -> of_core_type ct1 ** of_core_type ct2
-  | Ttyp_tuple cts | Ttyp_constr (_, _, cts) | Ttyp_class (_, _, cts) ->
+  | Ttyp_tuple cts ->
+    list_fold of_core_type (List.map ~f:snd cts) (* todo labels *)
+  | Ttyp_constr (_, _, cts) | Ttyp_class (_, _, cts) ->
     list_fold of_core_type cts
   | Ttyp_object (cts, _) ->
     list_fold
       (fun of_ ->
-        match of_.of_desc with
-        | OTtag (_, ct) | OTinherit ct -> of_core_type ct)
+         match of_.of_desc with
+         | OTtag (_, ct) | OTinherit ct -> of_core_type ct)
       cts
   | Ttyp_poly (_, ct) | Ttyp_alias (ct, _) -> of_core_type ct
   | Ttyp_variant (rfs, _, _) -> list_fold (fun rf -> app (Row_field rf)) rfs
@@ -588,9 +596,9 @@ let of_node = function
   | Structure { str_items; str_final_env } ->
     list_fold_with_next
       (fun next item ->
-        match next with
-        | None -> app (Structure_item (item, str_final_env))
-        | Some item' -> app (Structure_item (item, item'.str_env)))
+         match next with
+         | None -> app (Structure_item (item, str_final_env))
+         | Some item' -> app (Structure_item (item, item'.str_env)))
       str_items
   | Structure_item ({ str_desc }, _) -> of_structure_item_desc str_desc
   | Module_binding mb ->
@@ -601,9 +609,9 @@ let of_node = function
   | Signature { sig_items; sig_final_env } ->
     list_fold_with_next
       (fun next item ->
-        match next with
-        | None -> app (Signature_item (item, sig_final_env))
-        | Some item' -> app (Signature_item (item, item'.sig_env)))
+         match next with
+         | None -> app (Signature_item (item, sig_final_env))
+         | Some item' -> app (Signature_item (item, item'.sig_env)))
       sig_items
   | Signature_item ({ sig_desc }, _) -> of_signature_item_desc sig_desc
   | Module_declaration md ->
@@ -617,13 +625,13 @@ let of_node = function
   | With_constraint (Twith_modtype mt | Twith_modtypesubst mt) ->
     of_module_type mt
   | Core_type { ctyp_desc } -> of_core_type_desc ctyp_desc
-  | Package_type { pack_fields } ->
-    list_fold (fun (_, ct) -> of_core_type ct) pack_fields
+  | Package_type { tpt_cstrs } ->
+    list_fold (fun (_, ct) -> of_core_type ct) tpt_cstrs
   | Row_field rf -> begin
-    match rf.rf_desc with
-    | Ttag (_, _, cts) -> list_fold of_core_type cts
-    | Tinherit ct -> of_core_type ct
-  end
+      match rf.rf_desc with
+      | Ttag (_, _, cts) -> list_fold of_core_type cts
+      | Tinherit ct -> of_core_type ct
+    end
   | Value_description { val_desc } -> of_core_type val_desc
   | Type_declaration { typ_params; typ_cstrs; typ_kind; typ_manifest } ->
     let of_typ_cstrs (ct1, ct2, _) = of_core_type ct1 ** of_core_type ct2 in
@@ -742,11 +750,11 @@ let fake_path { Location.loc; txt = lid } typ name =
 let pattern_paths (type k) { Typedtree.pat_desc; pat_extra; _ } =
   let init =
     match (pat_desc : k pattern_desc) with
-    | Tpat_construct (lid_loc, { Types.cstr_name; cstr_res; _ }, _, _) ->
+    | Tpat_construct (lid_loc, { Data_types.cstr_name; cstr_res; _ }, _, _) ->
       fake_path lid_loc cstr_res cstr_name
     | Tpat_var (id, { Location.loc; txt }, _uid) ->
       [ (mkloc (Path.Pident id) loc, Some (Longident.Lident txt)) ]
-    | Tpat_alias (_, id, loc, _uid) ->
+    | Tpat_alias (_, id, loc, _uid, _) ->
       [ (reloc (Path.Pident id) loc, Some (Longident.Lident loc.txt)) ]
     | _ -> []
   in
@@ -781,7 +789,7 @@ let expression_paths { Typedtree.exp_desc; exp_extra; _ } =
     | Texp_override (_, ps) ->
       List.map
         ~f:(fun (id, loc, _) ->
-          (reloc (Path.Pident id) loc, Some (Longident.Lident loc.txt)))
+            (reloc (Path.Pident id) loc, Some (Longident.Lident loc.txt)))
         ps
     | Texp_letmodule (Some id, loc, _, _, _) ->
       [ (reloc (Path.Pident id) loc, Option.map ~f:mk_lident loc.txt) ]
@@ -793,7 +801,7 @@ let expression_paths { Typedtree.exp_desc; exp_extra; _ } =
         | _ -> assert false
       in
       [ (mkloc (Path.Pident id) loc, lid) ]
-    | Texp_construct (lid_loc, { Types.cstr_name; cstr_res; _ }, _) ->
+    | Texp_construct (lid_loc, { Data_types.cstr_name; cstr_res; _ }, _) ->
       fake_path lid_loc cstr_res cstr_name
     | Texp_open (od, _) -> module_expr_paths od.open_expr
     | _ -> []
@@ -828,7 +836,7 @@ let structure_item_paths { Typedtree.str_desc } =
   | Tstr_class_type cls ->
     List.map
       ~f:(fun (id, loc, _) ->
-        (reloc (Path.Pident id) loc, Some (Longident.Lident loc.txt)))
+          (reloc (Path.Pident id) loc, Some (Longident.Lident loc.txt)))
       cls
   | Tstr_open od -> module_expr_paths od.open_expr
   | _ -> []
@@ -878,8 +886,8 @@ let node_paths_full =
     [ (reloc (Path.Pident mtd_id) mtd_name, Some (Lident mtd_name.txt)) ]
   | With_constraint c -> with_constraint_paths c
   | Core_type ct -> core_type_paths ct
-  | Package_type { pack_path; pack_txt } ->
-    [ (reloc pack_path pack_txt, Some pack_txt.txt) ]
+  | Package_type { tpt_path; tpt_txt } ->
+    [ (reloc tpt_path tpt_txt, Some tpt_txt.txt) ]
   | Value_description { val_id; val_name } ->
     [ (reloc (Path.Pident val_id) val_name, Some (Lident val_name.txt)) ]
   | Type_declaration { typ_id; typ_name } ->
@@ -895,15 +903,15 @@ let node_paths_full =
   | Class_declaration ci -> ci_paths ci
   | Class_description ci -> ci_paths ci
   | Class_type_declaration ci -> ci_paths ci
-  | Record_field (_, { Types.lbl_res; lbl_name; _ }, lid_loc) ->
+  | Record_field (_, { Data_types.lbl_res; lbl_name; _ }, lid_loc) ->
     fake_path lid_loc lbl_res lbl_name
   | _ -> []
 
 let node_paths t = List.map (node_paths_full t) ~f:fst
 let node_paths_and_longident t =
   List.filter_map (node_paths_full t) ~f:(function
-    | _, None -> None
-    | p, Some lid -> Some (p, lid))
+      | _, None -> None
+      | p, Some lid -> Some (p, lid))
 
 let node_is_constructor = function
   | Constructor_declaration decl ->
