@@ -92,7 +92,7 @@ type node =
   | Method_call of expression * meth * Location.t
   | Record_field of
       [ `Expression of expression | `Pattern of pattern ]
-      * Types.label_description
+      * Data_types.label_description
       * Longident.t Location.loc
   | Module_binding_name of module_binding
   | Module_declaration_name of module_declaration
@@ -115,7 +115,7 @@ val node_paths_and_longident : node -> (Path.t Location.loc * Longident.t) list
 
 val node_is_constructor :
   node ->
-  [ `Description of Types.constructor_description
+  [ `Description of Data_types.constructor_description
   | `Declaration of Typedtree.constructor_declaration
   | `Extension_constructor of Typedtree.extension_constructor ]
   Location.loc
