@@ -24,6 +24,8 @@
 type space_formatter = (unit, Format.formatter, unit) format
 
 val longident : Format.formatter -> Longident.t -> unit
+val constr : Format.formatter -> Longident.t -> unit
+
 val expression : Format.formatter -> Parsetree.expression -> unit
 val string_of_expression : Parsetree.expression -> string
 
@@ -61,6 +63,24 @@ val tyvar: Format.formatter -> string -> unit
       position, or for keywords by escaping them with \#. No-op on "_". *)
 
 (* merlin *)
+type longindent_kind =
+| Constr (** variant constructors *)
+| Type (** core types, module types, class types, and classes *)
+| Other (** values and modules *)
+
 val case_list : Format.formatter -> Parsetree.case list -> unit
 val ident_of_name : Format.formatter -> string -> unit
-val needs_parens : string -> bool
+val needs_parens : kind:longindent_kind -> string -> bool
+
+
+(** {!Format_doc} functions for error messages *)
+module Doc:sig
+  val longident: Longident.t Format_doc.printer
+  val constr: Longident.t Format_doc.printer
+  val tyvar: string Format_doc.printer
+
+  (** Returns a format document if the expression reads nicely as the subject
+      of a sentence in a error message. *)
+  val nominal_exp : Parsetree.expression -> Format_doc.t option
+end
+
