@@ -15,7 +15,9 @@
 
 module Style = Misc.Style
 module Fmt = Format_doc
-
+type inclusion_env = Includemod.Functor_inclusion_diff.inclusion_env =
+  { i_env:Env.t; i_subst:Subst.t }
+  
 module Context = struct
   type pos =
     | Module of Ident.t
@@ -607,7 +609,7 @@ module Functor_suberror = struct
       Fmt.pp_open_tbox ()
       Diffing.prefix (pos, Diffing.classify diff)
       Fmt.pp_set_tab ()
-      (Printtyp.wrap_printing_env env ~error:true
+      (Printtyp.wrap_printing_env env.i_env ~error:true
          (fun () -> sub ~expansion_token env diff)
       )
      Fmt.pp_close_tbox ()
@@ -615,7 +617,7 @@ module Functor_suberror = struct
   let onlycase sub ~expansion_token env (_, diff) =
     Location.msg "%a@[<hv 2>%t@]"
       Fmt.pp_print_tab ()
-      (Printtyp.wrap_printing_env env ~error:true
+      (Printtyp.wrap_printing_env env.i_env ~error:true
          (fun () -> sub ~expansion_token env diff)
       )
 
@@ -689,7 +691,7 @@ let core env id x =
            "the first" "the second" env) diff.symptom
         show_locs (diff.got.val_loc, diff.expected.val_loc)
   | Err.Type_declarations diff ->
-      Fmt.dprintf "@[<v>@[<hv>%s:@;<1 2>%a@ %s@;<1 2>%a@]%a%a@]"
+      Fmt.dprintf "@[<v>@[<hv>%s:@;<1 2>%a@ %s@;<1 2>%a@]@,%a%a@]"
         "Type declarations do not match"
         !Oprint.out_sig_item
         (Out_type.tree_of_type_declaration id diff.got Trec_first)
@@ -773,6 +775,41 @@ let core_module_type_symptom (x:Err.core_module_type_symptom)  =
 
 (* Construct a linearized error message from the error tree *)
 
+let functor_expected ~before ~ctx =
+  let main =
+    (* The abstract module type case is detected by {!Includemod} *)
+    Fmt.dprintf
+      "@[This module should not be@ a@ structure,@ \
+       a@ functor@ was expected.@]"
+  in
+  dwith_context ctx main :: before
+
+let unexpected_functor ~env ~before ~ctx diff =
+  let rmty = diff.got.res in
+  let intro =
+    match diff.expected.res with
+    | Mty_ident _ ->
+        Fmt.dprintf
+          "@[This module should not be a functor,@ a@ module with an@ \
+           abstract@ module@ type@ was@ expected.@]"
+    | Mty_signature _ | _ ->
+        Fmt.dprintf
+          "@[This module should not be a functor,@ a@ structure was expected.@]"
+  in
+  let main =
+    match Includemod.modtypes_consistency ~loc:Location.none env rmty
+            diff.expected.res with
+    | _ ->
+        Fmt.dprintf
+          "%t@ @{<hint>Hint@}: Did you forget to apply the functor?"
+          intro
+    | exception _ ->
+        Fmt.dprintf "%t@ @[Moreover,@ the type of the functor@ body@ is@ \
+                     incompatible@ with@ the@ expected@ module type.@]"
+          intro
+  in
+  dwith_context ctx main :: before
+
 let rec module_type ~expansion_token ~eqmode ~env ~before ~ctx diff =
   match diff.symptom with
   | Invalid_module_alias _ (* the difference is non-informative here *)
@@ -814,8 +851,18 @@ and module_type_symptom ~eqmode ~expansion_token ~env ~before ~ctx = function
       in
       dwith_context ctx printer :: before
 
-and functor_params ~expansion_token ~env ~before ~ctx {got;expected;_} =
-  let d = Functor_suberror.Inclusion.patch env got expected in
+and functor_params ~expansion_token ~env ~before ~ctx diff =
+  match diff.got.params, diff.expected.params with
+  | [], _ -> functor_expected ~before ~ctx
+  | _, [] -> unexpected_functor ~env:env.i_env ~before ~ctx diff
+  | _ :: _, _ :: _ ->
+      compare_functor_params ~expansion_token ~env ~before ~ctx diff
+
+and compare_functor_params ~expansion_token ~env ~before ~ctx {got;expected;_} =
+  let d = Functor_suberror.Inclusion.patch env
+      (got.params, got.res)
+      (expected.params, expected.res)
+  in
   let actual = Functor_suberror.Inclusion.got d in
   let expected = Functor_suberror.expected d in
   let main =
@@ -849,12 +896,14 @@ and signature ~expansion_token ~env:_ ~before ~ctx sgs =
             :: before
           else
             before
-      | [], a :: _ -> sigitem ~expansion_token ~env:sgs.env ~before ~ctx a
+      | [], a :: _ ->
+          let env = {i_env=sgs.env; i_subst=sgs.subst } in
+          sigitem ~expansion_token ~env ~before ~ctx a
       | [], [] -> assert false
     )
 and sigitem ~expansion_token ~env ~before ~ctx (name,s) = match s with
   | Core c ->
-      dwith_context ctx (core env name c) :: before
+      dwith_context ctx (core env.i_env name c) :: before
   | Module_type diff ->
       module_type ~expansion_token ~eqmode:false ~env ~before
         ~ctx:(Context.Module name :: ctx) diff
@@ -886,7 +935,8 @@ and module_type_decl ~expansion_token ~env ~before ~ctx id diff =
       | None -> assert false
       | Some mty ->
           with_context (Modtype id::ctx)
-            (Runtime_coercion.illegal_permutation Context.alt_pp env) (mty,c)
+            (Runtime_coercion.illegal_permutation Context.alt_pp env.i_env)
+            (mty,c)
           :: before
       end
 
@@ -933,17 +983,17 @@ let module_type_subst ~env id diff =
         ~ctx:[Modtype id] mts.less_than
   | Illegal_permutation c ->
       let mty = diff.got in
-      let main =
-        with_context [Modtype id]
-          (Runtime_coercion.illegal_permutation Context.alt_pp env) (mty,c) in
-      [main]
+      [with_context [Modtype id]
+         (Runtime_coercion.illegal_permutation Context.alt_pp env.i_env)
+         (mty,c)
+      ]
 
 let all env = function
   | In_Compilation_unit diff ->
       let first = Location.msg "%a" interface_mismatch diff in
       signature ~expansion_token:true ~env ~before:[first] ~ctx:[] diff.symptom
   | In_Type_declaration (id,reason) ->
-      [Location.msg "%t" (core env id reason)]
+      [Location.msg "%t" (core env.i_env id reason)]
   | In_Module_type diff ->
       module_type ~expansion_token:true ~eqmode:false ~before:[] ~env ~ctx:[]
         diff
@@ -960,7 +1010,7 @@ let all env = function
 
 let err_msgs ppf (env, err) =
   Printtyp.wrap_printing_env ~error:true env
-    (fun () -> (coalesce @@ all env err)  ppf)
+    (fun () -> (coalesce @@ all {i_env=env; i_subst=Subst.identity} err) ppf)
 
 let report_error_doc err =
   Location.errorf
@@ -979,7 +1029,8 @@ let report_apply_error_doc ~loc env (app_name, mty_f, args) =
   | [ _, Change (g, e,  Err.Mismatch mty_diff) ] ->
       let more () =
         subcase_list @@
-        module_type_symptom ~eqmode:false ~expansion_token:true ~env ~before:[]
+        module_type_symptom ~eqmode:false ~expansion_token:true
+          ~env:{i_env=env; i_subst=Subst.identity} ~before:[]
           ~ctx:[] mty_diff.symptom
       in
       Location.errorf ~loc ~footnote "%t"
@@ -1019,6 +1070,7 @@ let report_apply_error_doc ~loc env (app_name, mty_f, args) =
         let actual = Functor_suberror.App.got d in
         let expected = Functor_suberror.expected d in
         let sub =
+          let env = {i_env=env; i_subst=Subst.identity} in
           List.rev @@
           Functor_suberror.params functor_app_diff env ~expansion_token:true d
         in
