@@ -70,6 +70,20 @@ type iterator =
 let iter_snd f (_, y) = f y
 let iter_loc sub {loc; _} = sub.location sub loc
 
+let rec iter_loc_lid sub lid =
+  let open Longident in
+  match lid with
+  | Lident _ -> ()
+  | Ldot (lid, id) ->
+      iter_loc sub lid; iter_loc_lid sub lid.txt; iter_loc sub id
+  | Lapply (lid, lid') ->
+      iter_loc sub lid; iter_loc_lid sub lid.txt;
+      iter_loc sub lid'; iter_loc_lid sub lid'.txt
+
+let iter_loc_lid sub {loc; txt} =
+  iter_loc sub {loc; txt};
+  iter_loc_lid sub txt
+
 let location _sub _l = ()
 
 let attribute sub x =
@@ -115,7 +129,7 @@ let module_substitution sub ms =
   sub.location sub ms_loc;
   sub.attributes sub ms_attributes;
   iter_loc sub ms_name;
-  iter_loc sub ms_txt
+  iter_loc_lid sub ms_txt
 
 let include_infos sub f {incl_loc; incl_mod; incl_attributes; _} =
   sub.location sub incl_loc;
@@ -211,7 +225,7 @@ let type_declarations sub (_, list) = List.iter (sub.type_declaration sub) list
 let type_extension sub x =
   sub.location sub x.tyext_loc;
   sub.attributes sub x.tyext_attributes;
-  iter_loc sub x.tyext_txt;
+  iter_loc_lid sub x.tyext_txt;
   List.iter (fun (c, _) -> sub.typ sub c) x.tyext_params;
   List.iter (sub.extension_constructor sub) x.tyext_constructors
 
@@ -231,15 +245,15 @@ let extension_constructor sub ec =
       List.iter (iter_loc sub) ids;
       constructor_args sub ctl;
       Option.iter (sub.typ sub) cto
-  | Text_rebind (_, lid) -> iter_loc sub lid
+  | Text_rebind (_, lid) -> iter_loc_lid sub lid
 
 let pat_extra sub (e, loc, attrs) =
   sub.location sub loc;
   sub.attributes sub attrs;
   match e with
-  | Tpat_type (_, lid) -> iter_loc sub lid
+  | Tpat_type (_, lid) -> iter_loc_lid sub lid
   | Tpat_unpack -> ()
-  | Tpat_open (_, lid, env) -> iter_loc sub lid; sub.env sub env
+  | Tpat_open (_, lid, env) -> iter_loc_lid sub lid; sub.env sub env
   | Tpat_constraint ct -> sub.typ sub ct
 
 let pat
@@ -253,17 +267,17 @@ let pat
   | Tpat_any  -> ()
   | Tpat_var (_, s, _) -> iter_loc sub s
   | Tpat_constant _ -> ()
-  | Tpat_tuple l -> List.iter (sub.pat sub) l
+  | Tpat_tuple l -> List.iter (fun (_, p) -> sub.pat sub p) l
   | Tpat_construct (lid, _, l, vto) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.pat sub) l;
       Option.iter (fun (ids, ct) ->
         List.iter (iter_loc sub) ids; sub.typ sub ct) vto
   | Tpat_variant (_, po, _) -> Option.iter (sub.pat sub) po
   | Tpat_record (l, _) ->
-      List.iter (fun (lid, _, i) -> iter_loc sub lid; sub.pat sub i) l
-  | Tpat_array l -> List.iter (sub.pat sub) l
-  | Tpat_alias (p, _, s, _) -> sub.pat sub p; iter_loc sub s
+      List.iter (fun (lid, _, i) -> iter_loc_lid sub lid; sub.pat sub i) l
+  | Tpat_array (_, l) -> List.iter (sub.pat sub) l
+  | Tpat_alias (p, _, s, _, _) -> sub.pat sub p; iter_loc sub s
   | Tpat_lazy p -> sub.pat sub p
   | Tpat_value p -> sub.pat sub (p :> pattern)
   | Tpat_exception p -> sub.pat sub p
@@ -306,7 +320,7 @@ let expr sub {exp_loc; exp_extra; exp_desc; exp_env; exp_attributes; _} =
   List.iter (fun (e, loc, _) -> extra e; sub.location sub loc) exp_extra;
   sub.env sub exp_env;
   match exp_desc with
-  | Texp_ident (_, lid, _)  -> iter_loc sub lid
+  | Texp_ident (_, lid, _)  -> iter_loc_lid sub lid
   | Texp_constant _ -> ()
   | Texp_let (rec_flag, list, exp) ->
       sub.value_bindings sub (rec_flag, list);
@@ -316,7 +330,10 @@ let expr sub {exp_loc; exp_extra; exp_desc; exp_env; exp_attributes; _} =
       function_body sub body
   | Texp_apply (exp, list) ->
       sub.expr sub exp;
-      List.iter (fun (_, o) -> Option.iter (sub.expr sub) o) list
+      List.iter (function
+        | (_, Arg exp) -> sub.expr sub exp
+        | (_, Omitted ()) -> ())
+        list
   | Texp_match (exp, cases, effs, _) ->
       sub.expr sub exp;
       List.iter (sub.case sub) cases;
@@ -325,25 +342,28 @@ let expr sub {exp_loc; exp_extra; exp_desc; exp_env; exp_attributes; _} =
       sub.expr sub exp;
       List.iter (sub.case sub) cases;
       List.iter (sub.case sub) effs
-  | Texp_tuple list -> List.iter (sub.expr sub) list
+  | Texp_tuple list -> List.iter (fun (_, e) -> sub.expr sub e) list
   | Texp_construct (lid, _, args) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.expr sub) args
   | Texp_variant (_, expo) -> Option.iter (sub.expr sub) expo
   | Texp_record { fields; extended_expression; _} ->
       Array.iter (function
         | _, Kept _ -> ()
-        | _, Overridden (lid, exp) -> iter_loc sub lid; sub.expr sub exp)
+        | _, Overridden (lid, exp) -> iter_loc_lid sub lid; sub.expr sub exp)
         fields;
       Option.iter (sub.expr sub) extended_expression;
   | Texp_field (exp, lid, _) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       sub.expr sub exp
   | Texp_setfield (exp1, lid, _, exp2) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       sub.expr sub exp1;
       sub.expr sub exp2
-  | Texp_array list -> List.iter (sub.expr sub) list
+  | Texp_atomic_loc (exp, lid, _) ->
+      iter_loc sub lid;
+      sub.expr sub exp
+  | Texp_array (_, list) -> List.iter (sub.expr sub) list
   | Texp_ifthenelse (exp1, exp2, expo) ->
       sub.expr sub exp1;
       sub.expr sub exp2;
@@ -360,7 +380,7 @@ let expr sub {exp_loc; exp_extra; exp_desc; exp_env; exp_attributes; _} =
       sub.expr sub exp3
   | Texp_send (exp, _) ->
       sub.expr sub exp
-  | Texp_new (_, lid, _) -> iter_loc sub lid
+  | Texp_new (_, lid, _) -> iter_loc_lid sub lid
   | Texp_instvar (_, _, s) -> iter_loc sub s
   | Texp_setinstvar (_, _, s, exp) ->
       iter_loc sub s;
@@ -383,16 +403,16 @@ let expr sub {exp_loc; exp_extra; exp_desc; exp_env; exp_attributes; _} =
       List.iter (sub.binding_op sub) ands;
       sub.case sub body
   | Texp_unreachable -> ()
-  | Texp_extension_constructor (lid, _) -> iter_loc sub lid
+  | Texp_extension_constructor (lid, _) -> iter_loc_lid sub lid
   | Texp_open (od, e) ->
       sub.open_declaration sub od;
       sub.expr sub e
   | Texp_typed_hole -> ()
 
 
-let package_type sub {pack_fields; pack_txt; _} =
-  List.iter (fun (lid, p) -> iter_loc sub lid; sub.typ sub p) pack_fields;
-  iter_loc sub pack_txt
+let package_type sub {tpt_cstrs; tpt_txt; _} =
+  List.iter (fun (lid, p) -> iter_loc_lid sub lid; sub.typ sub p) tpt_cstrs;
+  iter_loc_lid sub tpt_txt
 
 let binding_op sub {bop_loc; bop_op_name; bop_exp; _} =
   sub.location sub bop_loc;
@@ -436,8 +456,8 @@ let module_type sub {mty_loc; mty_desc; mty_env; mty_attributes; _} =
   sub.attributes sub mty_attributes;
   sub.env sub mty_env;
   match mty_desc with
-  | Tmty_ident (_, lid) -> iter_loc sub lid
-  | Tmty_alias (_, lid) -> iter_loc sub lid
+  | Tmty_ident (_, lid) -> iter_loc_lid sub lid
+  | Tmty_alias (_, lid) -> iter_loc_lid sub lid
   | Tmty_signature sg -> sub.signature sub sg
   | Tmty_functor (arg, mtype2) ->
       functor_parameter sub arg;
@@ -445,14 +465,14 @@ let module_type sub {mty_loc; mty_desc; mty_env; mty_attributes; _} =
   | Tmty_with (mtype, list) ->
       sub.module_type sub mtype;
       List.iter (fun (_, lid, e) ->
-        iter_loc sub lid; sub.with_constraint sub e) list
+        iter_loc_lid sub lid; sub.with_constraint sub e) list
   | Tmty_typeof mexpr -> sub.module_expr sub mexpr
 
 let with_constraint sub = function
   | Twith_type      decl -> sub.type_declaration sub decl
   | Twith_typesubst decl -> sub.type_declaration sub decl
-  | Twith_module    (_, lid) -> iter_loc sub lid
-  | Twith_modsubst  (_, lid) -> iter_loc sub lid
+  | Twith_module    (_, lid) -> iter_loc_lid sub lid
+  | Twith_modsubst  (_, lid) -> iter_loc_lid sub lid
   | Twith_modtype      mty -> sub.module_type sub mty
   | Twith_modtypesubst mty -> sub.module_type sub mty
 
@@ -460,7 +480,7 @@ let with_constraint sub = function
 let open_description sub {open_loc; open_expr; open_env; open_attributes; _} =
   sub.location sub open_loc;
   sub.attributes sub open_attributes;
-  iter_snd (iter_loc sub) open_expr;
+  iter_snd (iter_loc_lid sub) open_expr;
   sub.env sub open_env
 
 let open_declaration sub {open_loc; open_expr; open_env; open_attributes; _} =
@@ -490,7 +510,7 @@ let module_expr sub {mod_loc; mod_desc; mod_env; mod_attributes; _} =
   sub.env sub mod_env;
   match mod_desc with
   | Tmod_typed_hole  -> ()
-  | Tmod_ident (_, lid) -> iter_loc sub lid
+  | Tmod_ident (_, lid) -> iter_loc_lid sub lid
   | Tmod_structure st -> sub.structure sub st
   | Tmod_functor (arg, mexpr) ->
       functor_parameter sub arg;
@@ -532,13 +552,16 @@ let class_expr sub {cl_loc; cl_desc; cl_env; cl_attributes; _} =
       sub.class_expr sub cl
   | Tcl_apply (cl, args) ->
       sub.class_expr sub cl;
-      List.iter (fun (_, e) -> Option.iter (sub.expr sub) e) args
+      List.iter (function
+        | (_, Arg exp) -> sub.expr sub exp
+        | (_, Omitted ()) -> ())
+        args
   | Tcl_let (rec_flag, value_bindings, ivars, cl) ->
       sub.value_bindings sub (rec_flag, value_bindings);
       List.iter (fun (_, e) -> sub.expr sub e) ivars;
       sub.class_expr sub cl
   | Tcl_ident (_, lid, tyl) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.typ sub) tyl
   | Tcl_open (od, e) ->
       sub.open_description sub od;
@@ -551,7 +574,7 @@ let class_type sub {cltyp_loc; cltyp_desc; cltyp_env; cltyp_attributes; _} =
   match cltyp_desc with
   | Tcty_signature csg -> sub.class_signature sub csg
   | Tcty_constr (_, lid, list) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.typ sub) list
   | Tcty_arrow (_, ct, cl) ->
       sub.typ sub ct;
@@ -586,20 +609,20 @@ let typ sub {ctyp_loc; ctyp_desc; ctyp_env; ctyp_attributes; _} =
   | Ttyp_arrow (_, ct1, ct2) ->
       sub.typ sub ct1;
       sub.typ sub ct2
-  | Ttyp_tuple list -> List.iter (sub.typ sub) list
+  | Ttyp_tuple list -> List.iter (fun (_, t) -> sub.typ sub t) list
   | Ttyp_constr (_, lid, list) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.typ sub) list
   | Ttyp_object (list, _) -> List.iter (sub.object_field sub) list
   | Ttyp_class (_, lid, list) ->
-      iter_loc sub lid;
+      iter_loc_lid sub lid;
       List.iter (sub.typ sub) list
   | Ttyp_alias (ct, _) -> sub.typ sub ct
   | Ttyp_variant (list, _, _) -> List.iter (sub.row_field sub) list
   | Ttyp_poly (_, ct) -> sub.typ sub ct
   | Ttyp_package pack -> sub.package_type sub pack
   | Ttyp_open (_, mod_ident, t) ->
-      iter_loc sub mod_ident;
+      iter_loc_lid sub mod_ident;
       sub.typ sub t
 
 let class_structure sub {cstr_self; cstr_fields; _} =
