@@ -77,6 +77,22 @@ let tuple2 f1 f2 (x, y) = (f1 x, f2 y)
 let tuple3 f1 f2 f3 (x, y, z) = (f1 x, f2 y, f3 z)
 let map_loc sub {loc; txt} = {loc=sub.location sub loc; txt}
 
+let rec map_loc_lid sub lid =
+  let open Longident in
+  match lid with
+  | Lident id -> Lident id
+  | Ldot (lid, id) ->
+      let lid = { lid with txt = map_loc_lid sub lid.txt } in
+      Ldot (map_loc sub lid, map_loc sub id)
+  | Lapply (lid, lid') ->
+    let lid = { lid with txt = map_loc_lid sub lid.txt } in
+    let lid' = { lid' with txt = map_loc_lid sub lid'.txt } in
+     Lapply(map_loc sub lid, map_loc sub lid')
+
+let map_loc_lid sub {loc; txt} =
+  let txt = map_loc_lid sub txt in
+  map_loc sub {loc; txt}
+
 let location _sub l = l
 
 let attribute sub x =
@@ -125,7 +141,7 @@ let module_declaration sub x =
 let module_substitution sub x =
   let ms_loc = sub.location sub x.ms_loc in
   let ms_name = map_loc sub x.ms_name in
-  let ms_txt = map_loc sub x.ms_txt in
+  let ms_txt = map_loc_lid sub x.ms_txt in
   let ms_attributes = sub.attributes sub x.ms_attributes in
   {x with ms_loc; ms_name; ms_txt; ms_attributes}
 
@@ -227,7 +243,7 @@ let type_declarations sub (rec_flag, list) =
 
 let type_extension sub x =
   let tyext_loc = sub.location sub x.tyext_loc in
-  let tyext_txt = map_loc sub x.tyext_txt in
+  let tyext_txt = map_loc_lid sub x.tyext_txt in
   let tyext_params = List.map (tuple2 (sub.typ sub) id) x.tyext_params in
   let tyext_constructors =
     List.map (sub.extension_constructor sub) x.tyext_constructors
@@ -256,16 +272,16 @@ let extension_constructor sub x =
           Option.map (sub.typ sub) cto
         )
     | Text_rebind (path, lid) ->
-        Text_rebind (path, map_loc sub lid)
+        Text_rebind (path, map_loc_lid sub lid)
   in
   let ext_attributes = sub.attributes sub x.ext_attributes in
   {x with ext_loc; ext_name; ext_kind; ext_attributes}
 
 let pat_extra sub = function
   | Tpat_unpack as d -> d
-  | Tpat_type (path,loc) -> Tpat_type (path, map_loc sub loc)
-  | Tpat_open (path,loc,env) ->
-      Tpat_open (path, map_loc sub loc, sub.env sub env)
+  | Tpat_type (path,lid) -> Tpat_type (path, map_loc_lid sub lid)
+  | Tpat_open (path,lid,env) ->
+      Tpat_open (path, map_loc_lid sub lid, sub.env sub env)
   | Tpat_constraint ct -> Tpat_constraint (sub.typ sub ct)
 
 let pat
@@ -280,18 +296,20 @@ let pat
     | Tpat_any
     | Tpat_constant _ -> x.pat_desc
     | Tpat_var (id, s, uid) -> Tpat_var (id, map_loc sub s, uid)
-    | Tpat_tuple l -> Tpat_tuple (List.map (sub.pat sub) l)
-    | Tpat_construct (loc, cd, l, vto) ->
+    | Tpat_tuple l ->
+        Tpat_tuple (List.map (fun (label, p) -> label, sub.pat sub p) l)
+    | Tpat_construct (lid, cd, l, vto) ->
         let vto = Option.map (fun (vl,cty) ->
           List.map (map_loc sub) vl, sub.typ sub cty) vto in
-        Tpat_construct (map_loc sub loc, cd, List.map (sub.pat sub) l, vto)
+        Tpat_construct (map_loc_lid sub lid, cd, List.map (sub.pat sub) l, vto)
     | Tpat_variant (l, po, rd) ->
         Tpat_variant (l, Option.map (sub.pat sub) po, rd)
     | Tpat_record (l, closed) ->
-        Tpat_record (List.map (tuple3 (map_loc sub) id (sub.pat sub)) l, closed)
-    | Tpat_array l -> Tpat_array (List.map (sub.pat sub) l)
-    | Tpat_alias (p, id, s, uid) ->
-        Tpat_alias (sub.pat sub p, id, map_loc sub s, uid)
+        Tpat_record
+          (List.map (tuple3 (map_loc_lid sub) id (sub.pat sub)) l, closed)
+    | Tpat_array (mut, l) -> Tpat_array (mut, List.map (sub.pat sub) l)
+    | Tpat_alias (p, id, s, uid, ty) ->
+        Tpat_alias (sub.pat sub p, id, map_loc sub s, uid, ty)
     | Tpat_lazy p -> Tpat_lazy (sub.pat sub p)
     | Tpat_value p ->
        (as_computation_pattern (sub.pat sub (p :> pattern))).pat_desc
@@ -348,7 +366,7 @@ let expr sub x =
   let exp_desc =
     match x.exp_desc with
     | Texp_ident (path, lid, vd) ->
-        Texp_ident (path, map_loc sub lid, vd)
+        Texp_ident (path, map_loc_lid sub lid, vd)
     | Texp_constant _ as d -> d
     | Texp_let (rec_flag, list, exp) ->
         let (rec_flag, list) = sub.value_bindings sub (rec_flag, list) in
@@ -360,7 +378,7 @@ let expr sub x =
     | Texp_apply (exp, list) ->
         Texp_apply (
           sub.expr sub exp,
-          List.map (tuple2 id (Option.map (sub.expr sub))) list
+          List.map (tuple2 id (Typedtree.map_apply_arg (sub.expr sub))) list
         )
     | Texp_match (exp, cases, eff_cases, p) ->
         Texp_match (
@@ -376,16 +394,16 @@ let expr sub x =
           List.map (sub.case sub) eff_cases
         )
     | Texp_tuple list ->
-        Texp_tuple (List.map (sub.expr sub) list)
+        Texp_tuple (List.map (fun (label, e) -> label, sub.expr sub e) list)
     | Texp_construct (lid, cd, args) ->
-        Texp_construct (map_loc sub lid, cd, List.map (sub.expr sub) args)
+        Texp_construct (map_loc_lid sub lid, cd, List.map (sub.expr sub) args)
     | Texp_variant (l, expo) ->
         Texp_variant (l, Option.map (sub.expr sub) expo)
     | Texp_record { fields; representation; extended_expression } ->
         let fields = Array.map (function
             | label, Kept (t, mut) -> label, Kept (t, mut)
             | label, Overridden (lid, exp) ->
-                label, Overridden (map_loc sub lid, sub.expr sub exp))
+                label, Overridden (map_loc_lid sub lid, sub.expr sub exp))
             fields
         in
         Texp_record {
@@ -393,16 +411,18 @@ let expr sub x =
           extended_expression = Option.map (sub.expr sub) extended_expression;
         }
     | Texp_field (exp, lid, ld) ->
-        Texp_field (sub.expr sub exp, map_loc sub lid, ld)
+        Texp_field (sub.expr sub exp, map_loc_lid sub lid, ld)
     | Texp_setfield (exp1, lid, ld, exp2) ->
         Texp_setfield (
           sub.expr sub exp1,
-          map_loc sub lid,
+          map_loc_lid sub lid,
           ld,
           sub.expr sub exp2
         )
-    | Texp_array list ->
-        Texp_array (List.map (sub.expr sub) list)
+    | Texp_atomic_loc (exp, lid, ld) ->
+        Texp_atomic_loc (sub.expr sub exp, map_loc sub lid, ld)
+    | Texp_array (mut, list) ->
+        Texp_array (mut, List.map (sub.expr sub) list)
     | Texp_ifthenelse (exp1, exp2, expo) ->
         Texp_ifthenelse (
           sub.expr sub exp1,
@@ -437,7 +457,7 @@ let expr sub x =
     | Texp_new (path, lid, cd) ->
         Texp_new (
           path,
-          map_loc sub lid,
+          map_loc_lid sub lid,
           cd
         )
     | Texp_instvar (path1, path2, id) ->
@@ -490,7 +510,7 @@ let expr sub x =
     | Texp_unreachable ->
         Texp_unreachable
     | Texp_extension_constructor (lid, path) ->
-        Texp_extension_constructor (map_loc sub lid, path)
+        Texp_extension_constructor (map_loc_lid sub lid, path)
     | Texp_open (od, e) ->
         Texp_open (sub.open_declaration sub od, sub.expr sub e)
     | Texp_typed_hole ->
@@ -501,10 +521,10 @@ let expr sub x =
 
 
 let package_type sub x =
-  let pack_txt = map_loc sub x.pack_txt in
-  let pack_fields = List.map
-    (tuple2 (map_loc sub) (sub.typ sub)) x.pack_fields in
-  {x with pack_txt; pack_fields}
+  let tpt_txt = map_loc_lid sub x.tpt_txt in
+  let tpt_cstrs = List.map
+    (tuple2 (map_loc_lid sub) (sub.typ sub)) x.tpt_cstrs in
+  {x with tpt_txt; tpt_cstrs}
 
 let binding_op sub x =
   let bop_loc = sub.location sub x.bop_loc in
@@ -567,15 +587,15 @@ let module_type sub x =
   let mty_env = sub.env sub x.mty_env in
   let mty_desc =
     match x.mty_desc with
-    | Tmty_ident (path, lid) -> Tmty_ident (path, map_loc sub lid)
-    | Tmty_alias (path, lid) -> Tmty_alias (path, map_loc sub lid)
+    | Tmty_ident (path, lid) -> Tmty_ident (path, map_loc_lid sub lid)
+    | Tmty_alias (path, lid) -> Tmty_alias (path, map_loc_lid sub lid)
     | Tmty_signature sg -> Tmty_signature (sub.signature sub sg)
     | Tmty_functor (arg, mtype2) ->
         Tmty_functor (functor_parameter sub arg, sub.module_type sub mtype2)
     | Tmty_with (mtype, list) ->
         Tmty_with (
           sub.module_type sub mtype,
-          List.map (tuple3 id (map_loc sub) (sub.with_constraint sub)) list
+          List.map (tuple3 id (map_loc_lid sub) (sub.with_constraint sub)) list
         )
     | Tmty_typeof mexpr ->
         Tmty_typeof (sub.module_expr sub mexpr)
@@ -588,12 +608,12 @@ let with_constraint sub = function
   | Twith_typesubst decl -> Twith_typesubst (sub.type_declaration sub decl)
   | Twith_modtype mty -> Twith_modtype (sub.module_type sub mty)
   | Twith_modtypesubst mty -> Twith_modtypesubst (sub.module_type sub mty)
-  | Twith_module (path, lid) -> Twith_module (path, map_loc sub lid)
-  | Twith_modsubst (path, lid) -> Twith_modsubst (path, map_loc sub lid)
+  | Twith_module (path, lid) -> Twith_module (path, map_loc_lid sub lid)
+  | Twith_modsubst (path, lid) -> Twith_modsubst (path, map_loc_lid sub lid)
 
 let open_description sub od =
   {od with open_loc = sub.location sub od.open_loc;
-           open_expr = tuple2 id (map_loc sub) od.open_expr;
+           open_expr = tuple2 id (map_loc_lid sub) od.open_expr;
            open_env = sub.env sub od.open_env;
            open_attributes = sub.attributes sub od.open_attributes}
 
@@ -624,8 +644,8 @@ let module_expr sub x =
   let mod_env = sub.env sub x.mod_env in
   let mod_desc =
     match x.mod_desc with
-    | Tmod_ident (path, lid) -> Tmod_ident (path, map_loc sub lid)
     | Tmod_typed_hole -> Tmod_typed_hole
+    | Tmod_ident (path, lid) -> Tmod_ident (path, map_loc_lid sub lid)
     | Tmod_structure st -> Tmod_structure (sub.structure sub st)
     | Tmod_functor (arg, mexpr) ->
         Tmod_functor (functor_parameter sub arg, sub.module_expr sub mexpr)
@@ -690,7 +710,7 @@ let class_expr sub x =
     | Tcl_apply (cl, args) ->
         Tcl_apply (
           sub.class_expr sub cl,
-          List.map (tuple2 id (Option.map (sub.expr sub))) args
+          List.map (tuple2 id (Typedtree.map_apply_arg (sub.expr sub))) args
         )
     | Tcl_let (rec_flag, value_bindings, ivars, cl) ->
         let (rec_flag, value_bindings) =
@@ -703,7 +723,7 @@ let class_expr sub x =
           sub.class_expr sub cl
         )
     | Tcl_ident (path, lid, tyl) ->
-        Tcl_ident (path, map_loc sub lid, List.map (sub.typ sub) tyl)
+        Tcl_ident (path, map_loc_lid sub lid, List.map (sub.typ sub) tyl)
     | Tcl_open (od, e) ->
         Tcl_open (sub.open_description sub od, sub.class_expr sub e)
   in
@@ -719,7 +739,7 @@ let class_type sub x =
     | Tcty_constr (path, lid, list) ->
         Tcty_constr (
           path,
-          map_loc sub lid,
+          map_loc_lid sub lid,
           List.map (sub.typ sub) list
         )
     | Tcty_arrow (label, ct, cl) ->
@@ -766,15 +786,16 @@ let typ sub x =
     | Ttyp_var _ as d -> d
     | Ttyp_arrow (label, ct1, ct2) ->
         Ttyp_arrow (label, sub.typ sub ct1, sub.typ sub ct2)
-    | Ttyp_tuple list -> Ttyp_tuple (List.map (sub.typ sub) list)
+    | Ttyp_tuple list ->
+        Ttyp_tuple (List.map (fun (label, t) -> label, sub.typ sub t) list)
     | Ttyp_constr (path, lid, list) ->
-        Ttyp_constr (path, map_loc sub lid, List.map (sub.typ sub) list)
+        Ttyp_constr (path, map_loc_lid sub lid, List.map (sub.typ sub) list)
     | Ttyp_object (list, closed) ->
         Ttyp_object ((List.map (sub.object_field sub) list), closed)
     | Ttyp_class (path, lid, list) ->
         Ttyp_class
           (path,
-           map_loc sub lid,
+           map_loc_lid sub lid,
            List.map (sub.typ sub) list
           )
     | Ttyp_alias (ct, s) ->
@@ -786,7 +807,7 @@ let typ sub x =
     | Ttyp_package pack ->
         Ttyp_package (sub.package_type sub pack)
     | Ttyp_open (path, mod_ident, t) ->
-        Ttyp_open (path, map_loc sub mod_ident, sub.typ sub t)
+        Ttyp_open (path, map_loc_lid sub mod_ident, sub.typ sub t)
   in
   let ctyp_attributes = sub.attributes sub x.ctyp_attributes in
   {x with ctyp_loc; ctyp_desc; ctyp_env; ctyp_attributes}
