@@ -94,6 +94,9 @@ val list_remove: 'a -> 'a list -> 'a list
 val split_last: 'a list -> 'a list * 'a
        (** Return the last element and the other elements of the given list. *)
 
+val repeated_label : (string option * 'a) list -> string option
+       (** Detects a repeated label - for use with labeled tuples. *)
+
 val may: ('a -> unit) -> 'a option -> unit
 val may_map: ('a -> 'b) -> 'a option -> 'b option
 
@@ -300,18 +303,59 @@ val spellcheck : string list -> string -> string list
     list of suggestions taken from [env], that are close enough to
     [name] that it may be a typo for one of them. *)
 
-val did_you_mean :
-    Format_doc.formatter -> (unit -> string list) -> unit
-(** [did_you_mean ppf get_choices] hints that the user may have meant
-    one of the option returned by calling [get_choices]. It does nothing
-    if the returned list is empty.
 
-    The [unit -> ...] thunking is meant to delay any potentially-slow
-    computation (typically computing edit-distance with many things
-    from the current environment) to when the hint message is to be
-    printed. You should print an understandable error message before
-    calling [did_you_mean], so that users get a clear notification of
-    the failure even if producing the hint is slow.
+val align_hint:
+  prefix:string -> main:Format_doc.t -> hint:Format_doc.t ->
+  Format_doc.t * Format_doc.t
+(** [aligned_hint main hint] vertically aligns a [main] message and a hint
+    message. The vertical alignment is controlled by the use of [@{<ralign> ...
+    @}] boxes: the start of one box, in either the hint or the main message,
+    will be shifted on the left to ensure that the end of the two boxes are
+    vertically aligned, taking in account a pre-existing [prefix] before the
+    main message. For instance,
+{[
+let main, sub =
+  align_hint
+    ~prefix:"Error: "
+    (doc_printf "@{<ralign>The value @}%a is not an instance variable"
+      Style.inline_code "foobar"
+    )
+    (doc_printf
+      "@{<ralign>Did you mean @}%a" Style.inline_code "foobaz"
+    ) in
+   printf "Error: %a%a" pp_doc main pp_doc sub
+]}
+
+   produces the following text:
+
+{[
+Error:   The value "foobaz" is not an instance variable
+Hint: Did you mean "foobar"?
+]}
+
+  where the main message has been shifted to the left to align ["foobaz"] and
+  ["foobar"].
+*)
+
+
+val align_error_hint:
+  main:Format_doc.t -> hint:Format_doc.t -> Format_doc.t * Format_doc.t
+(** Same as [align_hint ~prefix:"Error: "] *)
+
+val aligned_hint:
+  prefix:string -> Format_doc.formatter ->
+  ('a, Format_doc.formatter, unit, Format_doc.t option -> unit) format4 -> 'a
+(** [aligned_hint ~prefix fmt ... hint] align the potential hint with the main
+    error message generated by the format string [fmt] before printing the two
+    message. *)
+
+val did_you_mean :
+    ?pp:string Format_doc.printer -> string list -> Format_doc.t option
+(** [did_you_mean ~pp choices] hints that the user may have meant one of the
+  option in [choices].
+
+  Each choice is printed with the [pp] function, or [Style.inline_code] if
+  [pp]=[None].
 *)
 
 val cut_at : string -> char -> string * string
@@ -364,7 +408,7 @@ module Color: sig
   type setting = Auto | Always | Never
 
   val default_setting : setting
-
+  val is_enabled : unit -> bool
 end
 
 
@@ -406,6 +450,7 @@ module Style : sig
     inline_code: tag_style;
   }
 
+  val hint: Format_doc.formatter -> unit
   val as_inline_code: 'a Format_doc.printer -> 'a Format_doc.printer
   val inline_code: string Format_doc.printer
 
@@ -422,6 +467,16 @@ module Style : sig
   (* adds functions to support color tags to the given formatter. *)
 end
 
+(* See the -error-style option *)
+module Error_style : sig
+  type setting =
+    | Contextual
+    | Short
+    | Merlin
+
+  val default_setting : setting
+end
+
 val print_see_manual : int list Format_doc.printer
 (** See manual section *)
 
