@@ -257,6 +257,10 @@ module List = struct
         | Some a' -> Cons (a', lazy (filter_map ~f (Lazy.force tl))))
   end
 
+  let hd_opt = function
+    | [] -> None
+    | x :: _ -> Some x
+
   let rec last = function
     | [] -> None
     | [ x ] -> Some x
@@ -290,6 +294,18 @@ module List = struct
   let sort_uniq ~cmp l = dedup_adjacent ~cmp (sort ~cmp l)
 
   let print f () l = "[ " ^ String.concat "; " (List.map (f ()) l) ^ " ]"
+
+  let max_elt list ~cmp =
+    fold_left list ~init:None ~f:(fun acc elt ->
+        match acc with
+        | None -> Some elt
+        | Some max -> if cmp max elt < 0 then Some elt else acc)
+
+  let min_elt list ~cmp =
+    fold_left list ~init:None ~f:(fun acc elt ->
+        match acc with
+        | None -> Some elt
+        | Some min -> if cmp min elt > 0 then Some elt else acc)
 end
 
 module Option = struct
@@ -327,6 +343,8 @@ module Option = struct
     let return x = Some x
     let ( >>= ) x f = bind x ~f
     let ( >>| ) x f = map x ~f
+    let ( let* ) opt f = bind opt ~f
+    let ( let+ ) opt f = map opt ~f
   end
 
   include Infix
